import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import ddf.minim.spi.*; 
import ddf.minim.signals.*; 
import ddf.minim.*; 
import ddf.minim.analysis.*; 
import ddf.minim.ugens.*; 
import ddf.minim.effects.*; 
import java.awt.Image; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class A1_LD_OneScreen extends PApplet {









Minim minim;
PFont font;
ArrayList<Entity> entities = new ArrayList<Entity>();
PImage[] sprites = new PImage[16];
PImage background;
AudioPlayer song;
AudioSample[] sounds = new AudioSample[5];
boolean[] keys = new boolean[16];
final int SCALE = 3;
boolean gameOver = false;
boolean gameStarted = false;
float lastTime = 0;
float timeToStart = 0;
int score = 0;

public void setup() {
  size(256*SCALE, 256*SCALE);
  minim = new Minim(this);
  noSmooth();
  background = loadImage("Background.png");
  sprites[0] = loadImage("Ship.png");
  sprites[1] = loadImage("Enemyship.png");
  sprites[2] = loadImage("Bullet.png");
  sprites[3] = loadImage("Enemybullet.png");
  sprites[4] = loadImage("Ball.png");
  sounds[0] = minim.loadSample("PlayerShoot.wav");
  sounds[1] = minim.loadSample("PlayerDamage.wav");
  sounds[2] = minim.loadSample("EnemyShoot.wav");
  sounds[3] = minim.loadSample("EnemyDamage.wav");
  sounds[4] = minim.loadSample("GameOver.wav");
  song = minim.loadFile("Chiptune.wav");
  song.setGain(-5);
  for (int n = 0; n < sounds.length-1; n++) {
    sounds[n].setGain(-11);
  }
  font = createFont("font.ttf", 8);
  textFont(font);
  
  PImage icon = loadImage("Ship.png");
  frame.setIconImage((Image) icon.getNative());
  frame.setTitle("Pong Invaders!");
}


public void draw() {
  scale(SCALE);
  textSize(8);
  textAlign(LEFT, TOP);
  image(background, 0, 0);
  if (gameStarted == true) {
    if (gameOver == false) {
      if (millis()-lastTime > PApplet.parseInt(random(500))+1000 && entities.size() < 25 && millis()-timeToStart > 5000) {
        lastTime = millis();
        entities.add(new EnemyEntity(random(width/SCALE), -16));
      }
      for (int n = 0; n < entities.size (); n++) {
        Entity entity = entities.get(n);
        entity.update();
        if (entity.removed) entity.destroy();
      }
    }
    text("Score: " + score, 1, 10);
    text("HP: " + ((Player)entities.get(0)).hp, 1, 1);
  } else {
    textAlign(CENTER, TOP);
    textSize(16);
    text("Pong Invaders!", width/SCALE/2, 4);
    textSize(8);
    text("How To Play:", width/SCALE/2, 32);
    text("Keep the ball bouncing to earn points", width/SCALE/2, 48);
    text("Shoot the enemy ships to destroy them", width/SCALE/2, 60);
    text("If you miss the ball you will lose points", width/SCALE/2, 72);
    text("Controls:", width/SCALE/2, 96);
    text("Left and Right arrow keys to move", width/SCALE/2, 112);
    text("Spacebar to shoot", width/SCALE/2, 124);
    text("Press Any Key To Play", width/SCALE/2, height/SCALE-32);
  }
  if (gameOver == true) {
    textAlign(CENTER, TOP);
    textSize(16);
    text("Game Over", width/SCALE/2, height/SCALE/2-8);
    textSize(8);
    text("Click to Play Again", width/SCALE/2, height/SCALE-32);
    if(mousePressed) {
      resetGame();
    }
  }

  if (keyPressed == true && gameStarted == false) {
    startGame();
  }
}

public void keyPressed() {
  if (gameOver == false) {
    if (key == CODED) {
      if (keyCode == LEFT) keys[0] = true;
      if (keyCode == RIGHT) keys[1] = true;
      if (keyCode == UP) keys[2] = true;
      if (keyCode == DOWN) keys[3] = true;
    } else {
      if (key == ' ') keys[4] = true;
    }
  }
}

public void keyReleased() {
  if (key == CODED) {
    if (keyCode == LEFT) keys[0] = false;
    if (keyCode == RIGHT) keys[1] = false;
    if (keyCode == UP) keys[2] = false;
    if (keyCode == DOWN) keys[3] = false;
  } else {
    if (key == ' ') keys[4] = false;
  }
}

public void startGame() {
  gameStarted = true;
  gameOver = false;
  score = 0;
  lastTime = millis();
  timeToStart = millis();
  entities.removeAll(entities);
  entities.add(new Player(width/SCALE/2-8, height/SCALE-32));
  entities.add(new Ball(width/SCALE/2-8, height/SCALE/2, random(60)+15+(90*PApplet.parseInt(random(4)))));
  song.play();
  song.loop();
}

public void resetGame() {
  gameStarted = false;
  gameOver = false;
  score = 0;
  entities.removeAll(entities);
  song.rewind();
  song.pause();
}



public class Ball extends Entity {
  
  public float dir;
  public boolean ballLost;
  
  public Ball(float x, float y, float dir) {
    this.x = x;
    this.y = y;
    this.dir = dir;
    speed = 2;
    ballLost = false;
    xa = sin(radians(dir))*speed;
    ya = cos(radians(dir))*speed;
    fric = 1;
    size = 8;
    tex = 4;
  }
  
  public void update() {
    if(x < 0) {
      x = 0;
      xa=-xa;
    }
    if(x+size > width/SCALE) {
      x = width/SCALE-size;
      xa=-xa;
    }
    if(y < 0) {
      y = 0;
      ya=-ya;
    }
    if(y > height/SCALE) {
      ballLost = true;
    }
    super.update();
    if(ballLost == true) {
      resetBall();
    }
  }
  
  public void collide(Entity entity) {
    if(entity instanceof EnemyEntity) {
      ((EnemyEntity)entity).damage(2);
    }
    if(entity instanceof Player) {
      y = entity.y-size;
      ya=-ya;
      score+=5;
    }
    if(entity instanceof Bullet) {
      entity.removed = true;
    }
  }
  
  public void resetBall() {
    ballLost = false;
    score -= 25;
    this.dir = random(60)+15+(90*PApplet.parseInt(random(4)));
    xa = sin(radians(dir))*speed;
    ya = cos(radians(dir))*speed;
    x = width/SCALE/2;
    y = height/SCALE/2;
  }
}


public class Bullet extends Entity {
  protected int dmg;
  protected Entity owner;
  public Bullet(Entity owner, float x, float y, float xa, float ya, int dmg) {
    this.x = x;
    this.y = y;
    this.xa = xa;
    this.ya = ya;
    this.owner = owner;
    this.dmg = dmg;
    fric = 1;
    size = 4;
    tex = 2+owner.tex;
  }
  
  public void update() {
    super.update();
    if(!inWindow()) destroy();
  }
  
  public void collide(Entity entity) {
    super.collide(entity);
    if(this.owner.getClass() == entity.getClass()) {
      return;
    }
    if(entity instanceof EnemyEntity) {
      ((EnemyEntity)entity).damage(this.dmg);
      removed = true;
      score++;
    }
    if(entity instanceof Player) {
      ((Player)entity).damage(this.dmg);
      removed = true;
    }
  }
}
public class EnemyEntity extends Entity {
  public int hp;
  public float dir;
  public int shootTime = 0;
  public int hurtTime = 0;
  
  public EnemyEntity(float x, float y) {
    this.x = x;
    this.y = y;
    ya = 8;
    speed = 0.85f;
    tex = 1;
    fric = 0.85f;
    hp = 3;
    size = 17;
  }
  
  public void update() {
    if(shootTime > 0) shootTime--;
    if(hurtTime > 0) {
      tint(255,0,0);
      hurtTime--;
    } else {
      noTint();
    }
    if(xa == 0) {
      xa = (random(9)-4)*speed;
    }
    if(ya == 0) {
      ya = (random(5)-2)*speed;
    }
    if(!inWindow()) {
      xa = -xa;
      ya = -ya;
    }
    super.update();
    Player p = (Player)entities.get(0);
    if(((p.x > x && p.x < x+size) || (p.x+p.size > x && p.x+p.size < x+size)) && gameOver == false ) shoot();
  }
  
  public void shoot() {
    if(shootTime == 0) {
      sounds[2].trigger();
      shootTime = PApplet.parseInt(random(25)+50);
      entities.add(new Bullet(this, x, y, 0, 3, 1));
    }
  }
  
  public void collide(Entity entity) {
    if(entity instanceof EnemyEntity) {
      xa = -xa;
      x+=xa;
      ya = -ya;
      y+=ya;
    }
  }
  
  public void damage(int dmg) {
    if(hurtTime == 0) {
      sounds[3].trigger();
      hurtTime = 15;
      hp -= dmg;
      if(hp <= 0) removed = true;
    }
  }
}

public class Entity {

  public float x, y;
  public float xa = 0;
  public float ya = 0;
  public float speed = 1;
  public float fric = 0.75f;
  public int tex = 0;
  public int size = 1;
  public boolean removed = false;

  public void update() {
    x += xa;
    y += ya;

    for (Entity e : entities) {
      if (checkCollision(e)) collide(e);
    }

    xa *= fric;
    ya *= fric;

    if (abs(xa) < 0.0001f) xa = 0;
    if (abs(ya) < 0.0001f) ya = 0;
    image(sprites[tex], x, y);
    noTint();
  }

  public void collide(Entity entity) {
  }

  public boolean checkCollision(Entity e) {
    if (e != this) {
      for (int x1 = 0; x1 <= 1; x1++) {
        for (int y1 = 0; y1 <= 1; y1++) {
          if (x+size*x1 >= e.x && x+size*x1 <= e.x+e.size) {
            if (y+size*y1 >= e.y && y+size*y1 <= e.y+e.size) {
              return true;
            }
          }
        }
      }
    }
    return false;
  }

  public boolean inWindow() {
    if (x+size >= 0 && x < width/SCALE && y+size >= 0 && y < height/SCALE) {
      return true;
    }
    return false;
  }

  public void destroy() {
    entities.remove(this);
  }
}

public class Player extends Entity {
  public int hp;
  public int shootTime = 0;
  public int hurtTime = 0;

  public Player(float x, float y) {
    this.x = x;
    this.y = y;
    hp = 10;
    speed = 0.5f;
    fric = 0.85f;
    size = 17;
    tex = 0;
  }

  public void update() {
    if (shootTime > 0) shootTime--;
    if (hurtTime > 0) {
      tint(255, 0, 0);
      hurtTime--;
    } else {
      noTint();
    }
    if (keys[0]) {
      xa-=speed;
    }
    if (keys[1]) {
      xa+=speed;
    }
    if (keys[4]) {
      shoot();
    }
    if (!inWindow()) {
      x = min(max(x, -size), width/SCALE);
    }
    super.update();
  }

  public void collide(Entity entity) {
  }

  protected void shoot() {
    if (shootTime == 0) {
      sounds[0].trigger();
      shootTime = 15;
      entities.add(new Bullet(this, x+size/2-2, y, xa/4, -2.5f, 1));
    }
  }

  public void damage(int dmg) {
    if (hurtTime == 0) {
      sounds[1].trigger();
      hurtTime = 15;
      hp = max(hp-dmg, 0);
      if (hp <= 0) {
        gameOver = true;
        sounds[4].trigger();
      }
    }
  }
}

  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "A1_LD_OneScreen" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
