import ddf.minim.spi.*;
import ddf.minim.signals.*;
import ddf.minim.*;
import ddf.minim.analysis.*;
import ddf.minim.ugens.*;
import ddf.minim.effects.*;
import java.awt.Image;

Minim minim;
PFont font;
ArrayList<Entity> entities = new ArrayList<Entity>();
PImage[] sprites = new PImage[16];
PImage background;
AudioPlayer song;
AudioSample[] sounds = new AudioSample[5];
boolean[] keys = new boolean[16];
final int SCALE = 3;
boolean gameOver = false;
boolean gameStarted = false;
float lastTime = 0;
float timeToStart = 0;
int score = 0;

void setup() {
  size(256*SCALE, 256*SCALE);
  minim = new Minim(this);
  noSmooth();
  background = loadImage("Background.png");
  sprites[0] = loadImage("Ship.png");
  sprites[1] = loadImage("Enemyship.png");
  sprites[2] = loadImage("Bullet.png");
  sprites[3] = loadImage("Enemybullet.png");
  sprites[4] = loadImage("Ball.png");
  sounds[0] = minim.loadSample("PlayerShoot.wav");
  sounds[1] = minim.loadSample("PlayerDamage.wav");
  sounds[2] = minim.loadSample("EnemyShoot.wav");
  sounds[3] = minim.loadSample("EnemyDamage.wav");
  sounds[4] = minim.loadSample("GameOver.wav");
  song = minim.loadFile("Chiptune.wav");
  song.setGain(-5);
  for (int n = 0; n < sounds.length-1; n++) {
    sounds[n].setGain(-11);
  }
  font = createFont("font.ttf", 8);
  textFont(font);
  
  PImage icon = loadImage("Ship.png");
  frame.setIconImage((Image) icon.getNative());
  frame.setTitle("Pong Invaders!");
}


void draw() {
  scale(SCALE);
  textSize(8);
  textAlign(LEFT, TOP);
  image(background, 0, 0);
  if (gameStarted == true) {
    if (gameOver == false) {
      if (millis()-lastTime > int(random(500))+1000 && entities.size() < 25 && millis()-timeToStart > 5000) {
        lastTime = millis();
        entities.add(new EnemyEntity(random(width/SCALE), -16));
      }
      for (int n = 0; n < entities.size (); n++) {
        Entity entity = entities.get(n);
        entity.update();
        if (entity.removed) entity.destroy();
      }
    }
    text("Score: " + score, 1, 10);
    text("HP: " + ((Player)entities.get(0)).hp, 1, 1);
  } else {
    textAlign(CENTER, TOP);
    textSize(16);
    text("Pong Invaders!", width/SCALE/2, 4);
    textSize(8);
    text("How To Play:", width/SCALE/2, 32);
    text("Keep the ball bouncing to earn points", width/SCALE/2, 48);
    text("Shoot the enemy ships to destroy them", width/SCALE/2, 60);
    text("If you miss the ball you will lose points", width/SCALE/2, 72);
    text("Controls:", width/SCALE/2, 96);
    text("Left and Right arrow keys to move", width/SCALE/2, 112);
    text("Spacebar to shoot", width/SCALE/2, 124);
    text("Press Any Key To Play", width/SCALE/2, height/SCALE-32);
  }
  if (gameOver == true) {
    textAlign(CENTER, TOP);
    textSize(16);
    text("Game Over", width/SCALE/2, height/SCALE/2-8);
    textSize(8);
    text("Click to Play Again", width/SCALE/2, height/SCALE-32);
    if(mousePressed) {
      resetGame();
    }
  }

  if (keyPressed == true && gameStarted == false) {
    startGame();
  }
}

void keyPressed() {
  if (gameOver == false) {
    if (key == CODED) {
      if (keyCode == LEFT) keys[0] = true;
      if (keyCode == RIGHT) keys[1] = true;
      if (keyCode == UP) keys[2] = true;
      if (keyCode == DOWN) keys[3] = true;
    } else {
      if (key == ' ') keys[4] = true;
    }
  }
}

void keyReleased() {
  if (key == CODED) {
    if (keyCode == LEFT) keys[0] = false;
    if (keyCode == RIGHT) keys[1] = false;
    if (keyCode == UP) keys[2] = false;
    if (keyCode == DOWN) keys[3] = false;
  } else {
    if (key == ' ') keys[4] = false;
  }
}

void startGame() {
  gameStarted = true;
  gameOver = false;
  score = 0;
  lastTime = millis();
  timeToStart = millis();
  entities.removeAll(entities);
  entities.add(new Player(width/SCALE/2-8, height/SCALE-32));
  entities.add(new Ball(width/SCALE/2-8, height/SCALE/2, random(60)+15+(90*int(random(4)))));
  song.play();
  song.loop();
}

void resetGame() {
  gameStarted = false;
  gameOver = false;
  score = 0;
  entities.removeAll(entities);
  song.rewind();
  song.pause();
}

